function Get-HPOVManagedNode
{
<#
    .SYNOPSIS
     
        Gets managed node properties from HP OpenView
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com"

    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" -HPOVServer "hpov.domain.com"
		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'Node Name')]
    [string]$NodeName,
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Server name of HP OpenView Server')]
    [string]$HPOVServer = "hpov.domain.com",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Get Chars
	if ($NodeName.Length -gt "1")
	{
		$FirstString = $NodeName.ToCharArray()[0]
		$LastString = $NodeName.ToCharArray()[-1]
	}
	else
	{
		$FirstString = "$NodeName"
		$LastString = "Null"
	}
	
	# Get Node Information	
	if ($FirstString -eq "*" -or $LastString -eq "*")
	{
		# Get Node
		$NodeObjects = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -like "$NodeName" }
		
		if ($NodeObjects)
		{
			foreach ($NodeObject in $NodeObjects)
			{
				# Output Results
				$Properties = New-Object Psobject
				$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
				$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
				$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
				$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
				if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
				Write-Output $Properties
			}
		}
	}
	else
	{
		# Get Node
		$NodeObject = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -eq "$NodeName" }
		
		if ($NodeObject)
		{
			# Output Results
			$Properties = New-Object Psobject
			$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
			$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
			$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
			$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
			if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
			Write-Output $Properties
		}
	}
}

function Set-HPOVManagedNode
{
<#
    .SYNOPSIS
     
        Sets managed node properties from HP OpenView
		
    .EXAMPLE
     
        Set-HPOVManagedNode -Name "node.domain.com" -Owner "NewOwner"

    .EXAMPLE
     
        Set-HPOVManagedNode -Name "node.domain.com" -Owner "NewOwner" -HPOVServer "hpov.domain.com"
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" | Set-HPOVManagedNode -Owner "NewOwner"
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" -HPOVServer "hpov.domain.com" | Set-HPOVManagedNode -Owner "NewOwner"
		
    .EXAMPLE
     
        Set-HPOVManagedNode -Name "node.domain.com" -Owner "NewOwner" -CommunicationPath "192.168.0.1" -IPAddresses "192.168.0.1"
		
    .EXAMPLE
     
        Set-HPOVManagedNode -Name "node.domain.com" -HPOVServer "hpov.domain.com" -Owner "NewOwner" -CommunicationPath "192.168.0.1" -IPAddresses "192.168.0.1"
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" | Set-HPOVManagedNode -Owner "NewOwner" -CommunicationPath "192.168.0.1" -IPAddresses "192.168.0.1"
		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
    [Parameter(
	    Position = 0,
        ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
        Mandatory = $true,
        HelpMessage = 'Node Name')]
    [string]$NodeName,
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Owner Name')]
    [string]$Owner,
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Communication Path')]
    [string]$CommunicationPath,
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'IP Addresses')]
    [string]$IpAddresses,
	
    [Parameter(
		Position = 1,
		ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
        Mandatory = $false,
        HelpMessage = 'Server name of HP OpenView Server')]
    [string]$HPOVServer = "hpov.domain.com",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Get Chars
	if ($NodeName.Length -gt "1")
	{
		$FirstString = $NodeName.ToCharArray()[0]
		$LastString = $NodeName.ToCharArray()[-1]
	}
	else
	{
		$FirstString = "$NodeName"
		$LastString = "Null"
	}
	
	# Get Node Information	
	if ($FirstString -eq "*" -or $LastString -eq "*")
	{
		# Get Node
		$NodeObjects = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -like "$NodeName" }
		
		if ($NodeObjects)
		{
			foreach ($NodeObject in $NodeObjects)
			{
				if ($Owner)
				{
					# Set Primary Owner
					$NodeObject.PrimaryOwnerName = $Owner
				}
				
				if ($CommunicationPath)
				{
					# Set Primary Owner
					$NodeObject.CommunicationPath = $CommunicationPath
				}
				
				if ($IpAddresses)
				{
					# Set Primary Owner
					$NodeObject.IpAddresses = $IpAddresses
				}
				
				if ($Owner -or $CommunicationPath -or $IpAddresses)
				{
					# Apply Changes
					$ApplyChanges = $NodeObject.Put()
				}
				
				# Output Results
				$Properties = New-Object Psobject
				$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
				$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
				$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
				$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
				if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
				Write-Output $Properties
			}
		}
	}
	else
	{
		# Get Node
		$NodeObject = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -eq "$NodeName" }
		
		if ($NodeObject)
		{
			if ($Owner)
			{
				# Set Primary Owner
				$NodeObject.PrimaryOwnerName = $Owner
			}
			
			if ($CommunicationPath)
			{
				# Set Primary Owner
				$NodeObject.CommunicationPath = $CommunicationPath
			}
			
			if ($IpAddresses)
			{
				# Set Primary Owner
				$NodeObject.IpAddresses = $IpAddresses
			}
			
			if ($Owner -or $CommunicationPath -or $IpAddresses)
			{
				# Apply Changes
				$ApplyChanges = $NodeObject.Put()
			}
			
			# Output Results
			$Properties = New-Object Psobject
			$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
			$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
			$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
			$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
			if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
			Write-Output $Properties
		}
	}
}

function Get-HPOVPDTList
{
<#
    .SYNOPSIS
     
        Gets managed nodes in PDT

    .EXAMPLE
     
        Get-HPOVPDTList
		
    .EXAMPLE
     
        Get-HPOVPDTList -HPOVServer "hpov.domain.com"
		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Server name of HP OpenView Server')]
    [string]$HPOVServer = "hpov.domain.com",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Get Node
	$NodeObjects = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.IsInSchedOutage -eq "True" }
		
	if ($NodeObjects)
	{
		foreach ($NodeObject in $NodeObjects)
		{
			# Output Results
			$Properties = New-Object Psobject
			$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
			$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
			$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
			$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
			if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
			Write-Output $Properties
		}
	}
}

function Start-HPOVManagedNodePDT
{
<#
    .SYNOPSIS
     
        Starts PDT for managed node
		
    .EXAMPLE
     
        Start-HPOVManagedNodePDT -Name "node.domain.com"

    .EXAMPLE
     
        Start-HPOVManagedNodePDT -Name "node.domain.com" -HPOVServer "hpov.domain.com"
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" | Start-HPOVManagedNodePDT
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" -HPOVServer "hpov.domain.com" | Start-HPOVManagedNodePDT
		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
    [Parameter(
	    Position = 0,
        ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
        Mandatory = $true,
        HelpMessage = 'Node Name')]
    [string]$NodeName,
	
    [Parameter(
		Position = 1,
		ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
        Mandatory = $false,
        HelpMessage = 'Server name of HP OpenView Server')]
    [string]$HPOVServer = "hpov.domain.com",
	
    [Parameter(
        Mandatory = $false,
        HelpMessage = 'Reason for maintenance mode')]
    [string]$Description = "Maintenance",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Get Chars
	if ($NodeName.Length -gt "1")
	{
		$FirstString = $NodeName.ToCharArray()[0]
		$LastString = $NodeName.ToCharArray()[-1]
	}
	else
	{
		$FirstString = "$NodeName"
		$LastString = "Null"
	}
	
	# Get Node Information	
	if ($FirstString -eq "*" -or $LastString -eq "*")
	{
		# Get Node
		$NodeObjects = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -like "$NodeName" }
		
		if ($NodeObjects)
		{
			foreach ($NodeObject in $NodeObjects)
			{
				# Start PDT
				$NodeObject.DeleteMsgInMaintMode = $True
				$NodeObject.DeleteMsgInSchedOutage = $True
				$NodeObject.DisableHeartBeatInMaintMode = $True
				$NodeObject.DisableHeartBeatInSchedOutage = $True
				$NodeObject.IsInSchedOutage = $True
				$ApplyChanges = $NodeObject.Put()

				# Output Results
				$Properties = New-Object Psobject
				$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
				$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
				$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
				$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
				if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
				Write-Output $Properties
			}
		}
	}
	else
	{
		# Get Node
		$NodeObject = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -eq "$NodeName" }
		
		if ($NodeObject)
		{
			# Start PDT
			$NodeObject.DeleteMsgInMaintMode = $True
			$NodeObject.DeleteMsgInSchedOutage = $True
			$NodeObject.DisableHeartBeatInMaintMode = $True
			$NodeObject.DisableHeartBeatInSchedOutage = $True
			$NodeObject.IsInSchedOutage = $True
			$ApplyChanges = $NodeObject.Put()
			
			# Output Results
			$Properties = New-Object Psobject
			$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
			$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
			$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
			$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
			if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
			Write-Output $Properties
		}
	}
}

function Stop-HPOVManagedNodePDT
{
<#
    .SYNOPSIS
     
        Stops PDT for managed node
		
    .EXAMPLE
     
        Stop-HPOVManagedNodePDT -Name "node.domain.com"

    .EXAMPLE
     
        Stop-HPOVManagedNodePDT -Name "node.domain.com" -HPOVServer "hpov.domain.com"

    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" | Stop-HPOVManagedNodePDT
		
    .EXAMPLE
     
        Get-HPOVManagedNode -Name "node.domain.com" -HPOVServer "hpov.domain.com" | Stop-HPOVManagedNodePDT
		
#>

[CmdletBinding(SupportsShouldProcess = $true)]
param (
    [Parameter(
	    Position = 0,
        ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
		Mandatory = $true,
        HelpMessage = 'Node Name')]
    [string]$NodeName,
	
    [Parameter(
		Position = 1,
		ValueFromPipeline = $True,
        ValueFromPipelineByPropertyName = $True,
        Mandatory = $false,
        HelpMessage = 'Server name of HP OpenView Server')]
    [string]$HPOVServer = "hpov.domain.com",
	
	[Parameter(
        Mandatory = $false,
        HelpMessage = 'Debug Mode')]
    [switch]$DebugMode = $false
)
	# Enable Debug Mode
	if ($DebugMode)
	{
		$DebugPreference = "Continue"
	}
	else
	{
		$ErrorActionPreference = "silentlycontinue"
	}
	
	# Get Chars
	if ($NodeName.Length -gt "1")
	{
		$FirstString = $NodeName.ToCharArray()[0]
		$LastString = $NodeName.ToCharArray()[-1]
	}
	else
	{
		$FirstString = "$NodeName"
		$LastString = "Null"
	}
	
	# Get Node Information	
	if ($FirstString -eq "*" -or $LastString -eq "*")
	{
		# Get Node
		$NodeObjects = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -like "$NodeName" }
		
		if ($NodeObjects)
		{
			foreach ($NodeObject in $NodeObjects)
			{
				# Stop PDT
				$NodeObject.DeleteMsgInMaintMode = $True
				$NodeObject.DeleteMsgInSchedOutage = $True
				$NodeObject.DisableHeartBeatInMaintMode = $True
				$NodeObject.DisableHeartBeatInSchedOutage = $True
				$NodeObject.IsInSchedOutage = $False
				$ApplyChanges = $NodeObject.Put()
				
				# Output Results
				$Properties = New-Object Psobject
				$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
				$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
				$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
				$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
				if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
				Write-Output $Properties
			}
		}
	}
	else
	{
		# Get Node
		$NodeObject = Get-WmiObject -ComputerName "$HPOVServer" -Namespace root\HewlettPackard\OpenView\data -Class OV_ManagedNode | Where { $_.PrimaryNodeName -eq "$NodeName" }
		
		if ($NodeObject)
		{
			# Stop PDT
			$NodeObject.DeleteMsgInMaintMode = $True
			$NodeObject.DeleteMsgInSchedOutage = $True
			$NodeObject.DisableHeartBeatInMaintMode = $True
			$NodeObject.DisableHeartBeatInSchedOutage = $True
			$NodeObject.IsInSchedOutage = $False
			$ApplyChanges = $NodeObject.Put()
			
			# Output Results
			$Properties = New-Object Psobject
			$Properties | Add-Member Noteproperty NodeName $NodeObject.PrimaryNodeName
			$Properties | Add-Member Noteproperty OwnerName $NodeObject.PrimaryOwnerName
			$Properties | Add-Member Noteproperty CommunicationPath $NodeObject.CommunicationPath
			$Properties | Add-Member Noteproperty IpAddresses $NodeObject.IpAddresses
			if ($NodeObject.IsInSchedOutage -eq "True")	{ $Properties | Add-Member Noteproperty IsInSchedOutage "True" } else { $Properties | Add-Member Noteproperty IsInSchedOutage "False" }
			Write-Output $Properties
		}
	}
}